/********************************************************************************************************/
/**                                                                                                     */
/**    Macro servant  convertir une table de format LP en format MPS                                   */
/**                Tlcharger via le site SASONLINE                                                    */
/**                Modifier le :                                                                        */
/**            Marche  suivre : Exctuer au complet le programme                                       */
/**                                                                                                     */
/********************************************************************************************************/


/*****************************************************************************************/
/* Purpose: This program converts the problem in SAS LP format to MPS format             */
/*                                                                                       */
/* Input:                                                                                */
/*    lp:     the name of the input SAS LP format dataset                                */
/*    mps:    the full name (including the directory) of the output MPS format dataset   */
/*    format: SPARSE - the input SAS LP format dataset is in sparse format;              */
/*             DENSE - the input SAS LP format dataset is in dense format;               */
/*                                                                                       */
/* Note:                                                                                 */
/*    Types SOSEQ, SOSLE, BASIC, PRICESEN, RHSSEN are not currently supported by         */
/*    proc optlp or proc optmilp. All information related with these types are           */
/*    discarded by this program.                                                         */
/*                                                                                       */
/*****************************************************************************************/


%macro lp2mpsd(version, lp=, mps=, format=SPARSE);

%let _version=1.4;
%if &version ne %then %put LP2MPSD macro Version &_version;

%let notes=%sysfunc(getoption(NOTES));
options nonotes;

/* Check for newer version */
 %if %sysevalf(&sysver >= 8.2) %then %do;
  %let _notfound=0;
  filename _ver url 'http://ftp.sas.com/techsup/download/stat/versions.dat' termstr=crlf;
  data _null_;
    infile _ver end=_eof;
    input name:$15. ver;
    if upcase(name)="&sysmacroname" then do;
       call symput("_newver",ver); stop;
    end;
    if _eof then call symput("_notfound",1);
    run;
  %if &syserr ne 0 or &_notfound=1 %then
    %put &sysmacroname: Unable to check for newer version;
  %else %if %sysevalf(&_newver > &_version) %then %do;
    %put &sysmacroname: A newer version of the &sysmacroname macro is available.;
    %put %str(         ) You can get the newer version at this location:;
    %put %str(         ) http://support.sas.com/ctx/samples/index.jsp;
  %end;
 %end;


%macro lp2mps_sparse(lp=, mps=);

    /* indicate whether there are unsupported information; */
    %let unsupported=0;

    /* get variable name, MPS dataset field length; */
    proc contents data=&lp out=lp_contents (keep=name type length) noprint;
    run;

    data _null_;
        set lp_contents;
        retain namelength;
        call symput("var"||strip(vvalue(_N_)), name);
        call symput("nvar", _N_);
        name = strip(lowcase(name));
        if substr(name,1,4)="_col" or substr(name,1,4)="_row" or substr(name,1,3)="_id" then do;
            namelength2 = length;
            if namelength=. or namelength<namelength2 then
                namelength = namelength2;
            call symput("namelen", namelength);
        end;
        else if length(name)>=5 then do;
            if substr(name,1,5)="_coef" then do;
                call symput("coeflen", length);
            end;
        end;
    run;

    /* keep a copy of input SAS LP format dataset; */
    data lp;
        set &lp;
    run;

    /* convert to standard format with 4 fields
    /* and standard field names _type_, _row_, _col_, and _coef_ */
    %if &nvar=4 %then %do;
        proc datasets nolist;
            modify lp;
            %do i=1 %to &nvar;
                %if %lowcase(&&var&i) ne _col_
                and %lowcase(%substr(&&var&i,1,4))=_col %then %do;
                    rename &&var&i=_col_;
                %end;
                %else %if %lowcase(&&var&i) ne _row_
                and (%lowcase(%substr(&&var&i,1,4))=_row or %lowcase(%substr(&&var&i,1,3))=_id) %then %do;
                    rename &&var&i=_row_;
                %end;
                %else %if %length(&&var&i)>=5 %then %do;
                    %if %lowcase(&&var&i) ne _coef_
                    and %lowcase(%substr(&&var&i,1,5))=_coef %then %do;
                        rename &&var&i=_coef_;
                    %end;
                %end;
            %end;
        quit;
    %end;
    %else %do;
        %let nrecord=%eval((&nvar-2)/2);
        proc datasets nolist;
            modify lp;
            %do i=1 %to &nvar;
                %if %lowcase(&&var&i) ne _col_ and %lowcase(%substr(&&var&i,1,4))=_col %then %do;
                    rename &&var&i=_col_;
                %end;
            %end;
            %do k=1 %to &nrecord;
                %do i=1 %to &nvar;
                    %if %lowcase(&&var&i) ne _row&k._
                    and (%lowcase(%substr(&&var&i,1,4))=_row or %lowcase(%substr(&&var&i,1,3))=_id)
                    and %index(&&var&i,&k)>0 %then %do;
                        rename &&var&i=_row&k._;
                    %end;
                    %else %if %length(&&var&i)>=5 %then %do;
                        %if %lowcase(&&var&i) ne _coef&k._ and %lowcase(%substr(&&var&i,1,5))=_coef and %index(&&var&i,&k)>0 %then %do;
                            rename &&var&i=_coef&k._;
                        %end;
                    %end;
                %end;
            %end;
        quit;

        data lp_temp;
            set lp;
            keep _type_ _col_ _row_ _coef_;
            %do k=1 %to &nrecord;
                _row_ = _row&k._;
                _coef_ = _coef&k._;
                if _row_ ne "" or _coef_ ne . then output;
            %end;
        run;

        proc datasets nolist;
            delete lp;
            change lp_temp=lp;
        quit;
    %end;

    /* sort by _col_ */
    proc sort data=lp out=lp_temp;
        by _col_;
    run;
    proc datasets nolist;
        delete lp;
        change lp_temp=lp;
    quit;

    /* define MPS format dataset section titles */
    data name;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="NAME";
        field3="&lp";
        if &namelen>8 then field5="FREE";
        else field5="";
        field2=""; field4=.; field6=.;
    run;

    data rows_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="ROWS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data columns_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="COLUMNS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data rhs_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="RHS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data ranges_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="RANGES";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data bounds_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="BOUNDS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data sos_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="SOS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data endata;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="ENDATA";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data intorg;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field3="'MARKER'";
        field5="'INTORG'";
        field1=""; field2=""; field4=.; field6=.;
    run;

    data intend;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field3="'MARKER'";
        field5="'INTEND'";
        field1=""; field2=""; field4=.; field6=.;
    run;

    /* generate MPS sections from input dataset */
    data rows columns rhs ranges bounds sos integer;
        set lp;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        retain id_upperbd id_lowerbd id_fixed id_free id_integer id_binary id_rhs id_ranges id_object;
        _type_ = upcase(_type_);
        _col_ = upcase(_col_);
        _row_ = upcase(_row_);

        /*
        * skip comments;
        */
        if substr(_type_,1,1)="*" then delete;

        /*
        * process special column names "_RHS_","_RHSSEN_","_RANGE_"
        */
        if _col_ = "_RHS_" then do;
            id_rhs="_RHS_";
            field1="";
            field2="_RHS_";
            field3=_row_;
            field4=_coef_;
            field5=""; field6=.;
            output rhs;
        end;
        else if _col_ = "_RANGE_" then do;
            id_ranges="_RANGE_";
            field1="";
            field2="_RANGE_";
            field3=_row_;
            field4=_coef_;
            field5=""; field6=.;
            output ranges;
        end;
        else if _col_ = "_RHSSEN_" then do;
            call symput("unsupported", "1");
            delete;
        end;

        /*
        * process by row type;
        */

        /* process row definitions; */
        if _type_ in ("MAX", "MIN", "EQ", "LE", "GE") then do;
            field1="";
            if _type_="MAX" then field1="MAX";
            else if _type_="MIN" then field1="N";
            else if _type_="EQ" then field1="E";
            else if _type_="LE" then field1="L";
            else if _type_="GE" then field1="G";
            field2 = _row_;
            field3=""; field4=.; field5=""; field6=.;
            output rows;
            if _type_ in ("MAX", "MIN") and id_object="" then do;
                id_object = _row_;
                call symput("id_obj", _row_);
            end;
            if _col_ ne "" and _col_ not in ("_RHS_","_RHSSEN_","_RANGE_") then do;
                field1="";
                field2=_col_;
                field3=_row_;
                field4=_coef_;
                field5=""; field6=.;
                output columns;
            end;
            else if _col_ eq "" then do;
                if id_rhs="" then id_rhs="_RHS_";
                field1="";
                field2=id_rhs;
                field3=_row_;
                field4=_coef_;
                field5=""; field6=.;
                output rhs;
            end;
        end;
        /* process free rows; */
        else if _type_="FREE" then do;
            field1="L";         /* output to rows section */
            field2=_row_;
            field3=""; field4=.; field5=""; field6=.;
            output rows;
            if _col_ ne "_RHS_" then do;  /* set right-hand-side to be infinity */
                field1="";
                field2=id_rhs;
                field3=_row_;
                field4=1e+40;
                field5=""; field6=.;
                output rhs;
            end;
            if _col_ ne "" and _col_ not in ("_RHS_","_RHSSEN_","_RANGE_") then do;
                field1="";
                field2=_col_;
                field3=_row_;
                field4=_coef_;
                field5=""; field6=.;
                output columns;
            end;
        end;
        /* process free variable; */
        else if _type_="UNRSTRCT" or _row_=id_free then do;
            if _col_="" then do;
                id_free = _row_;
            end;
            else do;
                field1="FR";
                field2="BOUNDS";
                field3=_col_;
                field4=.;
                field5=""; field6=.;
                output bounds;
                if id_free="" then id_free=_row_;
            end;
        end;
        /* process variable upper bound; */
        else if _type_ in ("UPPERBD", "UPPER") or _row_=id_upperbd then do;
            if _col_="" then do;
                id_upperbd = _row_;
            end;
            else do;
                field1="UP";
                field2="BOUNDS";
                field3=_col_;
                field4=_coef_;
                field5=""; field6=.;
                output bounds;
                if id_upperbd="" then id_upperbd=_row_;
            end;
        end;
        /* process variable lower bound; */
        else if _type_ in ("LOWERBD", "LOWER") or _row_=id_lowerbd then do;
            if _col_="" then do;
                id_lowerbd = _row_;
            end;
            else do;
                field1="LO";
                field2="BOUNDS";
                field3=_col_;
                field4=_coef_;
                field5=""; field6=.;
                output bounds;
                if id_lowerbd="" then id_lowerbd=_row_;
            end;
        end;
        /* process fixed variable; */
        else if _type_="FIXED" or _row_=id_fixed then do;
            if _col_="" then do;
                id_fixed = _row_;
            end;
            else do;
                field1="FX";
                field2="BOUNDS";
                field3=_col_;
                field4=_coef_;
                field5=""; field6=.;
                output bounds;
                if id_fixed="" then id_fixed=_row_;
            end;
        end;
        /* process integer variable; */
        else if _type_ in ("INTEGER", "INT") or _row_=id_integer then do;
            if _col_="" then do;
                id_integer = _row_;
            end;
            else do;
                field1="";
                field2=_col_;
                field3=_row_;
                field4=_coef_;
                field5=""; field6=.;
                output integer;
                if id_integer="" then id_integer=_row_;
            end;
        end;
        /* process binary variable; */
        else if _type_="BINARY" or _row_=id_binary then do;
            if _col_="" then do;
                id_binary = _row_;
            end;
            else do;
                field1="BV";
                field2="BOUNDS";
                field3=_col_;
                field4=.;
                field5=""; field6=.;
                output bounds;
                if id_binary="" then id_binary=_row_;
            end;
        end;
        /* process rhs; */
        else if _type_="RHS" or (_col_=id_rhs and id_rhs ne "") then do;
            id_rhs = _col_;
            field1="";
            field2=_col_;
            field3=_row_;
            field4=_coef_;
            field5=""; field6=.;
            output rhs;
        end;
        /* process range; */
        else if _type_="RANGE" or (_col_=id_ranges and id_ranges ne "") then do;
            id_ranges = _col_;
            field1="";
            field2=_col_;
            field3=_row_;
            field4=_coef_;
            field5=""; field6=.;
            output ranges;
        end;
        /* process unsupported types */
        else if _type_ in ("SOSEQ","SOSLE","BASIC","PRICESEN","RHSSEN") then do;
            call symput("unsupported", "1");
        end;
        /* process observations without _type_ ; */
        else do;
            field1="";
            field2=_col_;
            field3=_row_;
            field4=_coef_;
            field5=""; field6=.;
            output columns;
        end;
    run;

    /* delete empty optional sections */
    proc sql noprint;
        select count(*) into :count from rhs;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete rhs;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from ranges;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete ranges;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from bounds;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete bounds;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from sos;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete sos;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from integer;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete integer;
        quit;
    %end;

    /* remove duplicate observations */
    proc sql noprint;
        create table rows2 as select distinct * from rows;
        %if %sysfunc(exist(rhs)) %then %do;
            create table rhs2 as select distinct * from rhs;
        %end;
        %if %sysfunc(exist(ranges)) %then %do;
            create table ranges2 as select distinct * from ranges;
        %end;
        %if %sysfunc(exist(bounds)) %then %do;
            create table bounds2 as select distinct * from bounds;
        %end;
        %if %sysfunc(exist(sos)) %then %do;
            create table sos2 as select distinct * from sos;
        %end;
        /* move constraint matrix coefficients related with integer variables to separate dataset */
        %if %sysfunc(exist(integer)) %then %do;
            create table columns2 as
                select distinct * from columns where field2 not in (select field2 from integer);
            create table columns3 as
                select distinct * from columns where field2 in (select field2 from integer);
        %end;
    quit;

    %if %sysfunc(exist(integer)) %then %do;
        proc datasets nolist;
            delete integer;
            change columns3=integer;
        quit;

        proc sql noprint;
            select count(*) into :count from integer;
        quit;
        %if &count=0 %then %do;
            proc datasets nolist;
                delete integer;
            quit;
        %end;
    %end;

    proc datasets nolist;
        delete rows;
        change rows2=rows;
        %if %sysfunc(exist(rhs)) %then %do;
            delete rhs;
            change rhs2=rhs;
        %end;
        %if %sysfunc(exist(ranges)) %then %do;
            delete ranges;
            change ranges2=ranges;
        %end;
        %if %sysfunc(exist(bounds)) %then %do;
            delete bounds;
            change bounds2=bounds;
        %end;
        %if %sysfunc(exist(sos)) %then %do;
            delete sos;
            change sos2=sos;
        %end;
        %if %sysfunc(exist(integer)) %then %do;
            delete columns;
            change columns2=columns;
        %end;
    quit;

    /* if there is any variable only defined by BOUNDS, then need to define it in COLUMNS section */
    %if %sysfunc(exist(bounds)) %then %do;
        proc sql noprint;
            create table bounds2 as
                select distinct * from bounds where field3 not in (select field2 from columns);
        quit;

        %if %sysfunc(exist(integer)) %then %do;
            proc sql noprint;
                create table bounds3 as
                    select distinct * from bounds2 where field3 not in (select field2 from integer);
            quit;

            proc datasets nolist;
                delete bounds2;
                change bounds3=bounds2;
            quit;
        %end;

        proc sql noprint;
            select count(*) into :count from bounds2;
        quit;
        %if &count=0 %then %do;
            proc datasets nolist;
                delete bounds2;
            quit;
        %end;

        %if %sysfunc(exist(bounds2)) %then %do;
            data columns2;
                set bounds2;
                field1="";
                field2=field3;
                field3="&id_obj";
                field4=0;
                field5="";
                field6=.;
            run;

            proc datasets nolist;
                delete bounds2;
            quit;
        %end;
    %end;

    /* generate MPS format dataset */
    options notes;
    data &mps;
        set name
            rows_title rows
            columns_title columns
            %if %sysfunc(exist(columns2)) %then columns2;
            %if %sysfunc(exist(integer)) %then intorg integer intend;
            %if %sysfunc(exist(rhs)) %then rhs_title rhs;
            %if %sysfunc(exist(ranges)) %then ranges_title ranges;
            %if %sysfunc(exist(bounds)) %then bounds_title bounds;
            %if %sysfunc(exist(sos)) %then sos_title sos;
            endata;
    run;
    options nonotes;

    /* delete temporary datasets */
    proc datasets nolist;
        delete lp;
        delete lp_contents;
        delete name;
        delete rows_title;
        delete columns_title;
        delete rhs_title;
        delete ranges_title;
        delete bounds_title;
        delete sos_title;
        delete endata;
        delete intorg;
        delete intend;
        delete rows;
        delete columns;
        %if %sysfunc(exist(columns2)) %then %do; delete columns2; %end;
        %if %sysfunc(exist(integer)) %then %do; delete integer; %end;
        %if %sysfunc(exist(rhs)) %then %do; delete rhs; %end;
        %if %sysfunc(exist(ranges)) %then %do; delete ranges; %end;
        %if %sysfunc(exist(bounds)) %then %do; delete bounds; %end;
        %if %sysfunc(exist(sos)) %then %do; delete sos; %end;
    quit;

    /* if there are unsupported information, issue a warning message */
    %if &unsupported=1 %then %do;
        options nonotes;
        data _null_;
            putlog 'WARNING: Types SOSEQ, SOSLE, BASIC, PRICESEN, RHSSEN are';
            putlog 'WARNING: not currently supported by MPS format dataset.';
            putlog 'WARNING: All information related with these types are discarded.';
        run;
        option &notes;
    %end;
%mend;


%macro lp2mps_dense(lp=, mps=);

    /* indicate whether there are unsupported information */
    %let unsupported=0;

    /* get variable name, MPS dataset field length */
    proc contents data=&lp out=lp_contents (keep=name type length) noprint;
    run;

    data _null_;
        set lp_contents;
        retain i namelength coeflength;
        name2 = strip(lowcase(name));
        if name2="_row_" or name2="_id_" then do;
            call symput("row", name);
            namelength2 = length;
            if namelength=. or namelength<namelength2 then
                namelength = namelength2;
            call symput("namelen", namelength);
        end;
        else if name2="_range_" then do;
            call symput("range", name);
        end;
        else if name2="_rhs_" then do;
            call symput("rhs", name);
        end;
        else if name2="_rhssen_" then do;
            call symput("rhssen", name);
            call symput("unsupported", "1");
        end;
        else if name2="_type_" then do;
            call symput("type", name);
        end;
        else do;
            i+1;
            call symput("var"||strip(vvalue(i)), name);
            call symput("nvar", i);
            namelength2 = length(name);
            if namelength=. or namelength<namelength2 then
                namelength = namelength2;
            call symput("namelen", namelength);
        end;
        if type=1 then do;
            coeflength2 = length;
            if coeflength=. or coeflength<coeflength2 then
                coeflength = coeflength2;
            call symput("coeflen", coeflength);
        end;
    run;
    %if &namelen<8 %then %let namelen=8;

    /* define MPS format dataset section titles */
    data name;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="NAME";
        field3="&lp";
        if &namelen>8 then field5="FREE";
        else field5="";
        field2=""; field4=.; field6=.;
    run;

    data rows_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="ROWS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data columns_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="COLUMNS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data rhs_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="RHS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data ranges_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="RANGES";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data bounds_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="BOUNDS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data sos_title;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="SOS";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data endata;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field1="ENDATA";
        field2=""; field3=""; field4=.; field5=""; field6=.;
    run;

    data intorg;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field3="'MARKER'";
        field5="'INTORG'";
        field1=""; field2=""; field4=.; field6=.;
    run;

    data intend;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        field3="'MARKER'";
        field5="'INTEND'";
        field1=""; field2=""; field4=.; field6=.;
    run;

    /* generate MPS sections from input dataset */
    data rows columns rhs ranges bounds sos integer;
        set &lp;
        keep field1 field2 field3 field4 field5 field6;
        length field1 $8. field2 $&namelen. field3 $&namelen. field4 &coeflen. field5 $&namelen. field6 &coeflen.;
        retain numobj numrow;

        /* generate row name */
        _type_ = upcase(&type);
        %if %symexist(row) %then %do; _row_=&row; %end;
        %else %do;
            if _type_="MAX" or _type_="MIN" then do;
                numobj+1;
                nobs = strip(vvalue(numobj));
                nrepeat = 4 - length(nobs);
                _row_="OBJ"||repeat('0',nrepeat)||nobs;
            end;
            else do;
                numrow+1;
                nobs = strip(vvalue(numrow));
                nrepeat = 4 - length(nobs);
                _row_="ROW"||repeat('0',nrepeat)||nobs;
            end;
        %end;

        /*
        * process special column names "_RHS_","_RHSSEN_","_RANGE_"
        */
        %if %symexist(rhs) %then %do;
            field1="";
            field2="&rhs";
            field3=_row_;
            field4=&rhs;
            field5=""; field6=.;
            if &rhs ne . then output rhs;
        %end;
        %if %symexist(range) %then %do;
            field1="";
            field2="&range";
            field3=_row_;
            field4=&range;
            field5=""; field6=.;
            if &range ne . then output ranges;
        %end;

        /*
        * process by row type;
        */

        /* process row definitions; */
        if _type_ in ("MAX", "MIN", "EQ", "LE", "GE") then do;
            field1="";
            if _type_="MAX" then field1="MAX";
            else if _type_="MIN" then field1="N";
            else if _type_="EQ" then field1="E";
            else if _type_="LE" then field1="L";
            else if _type_="GE" then field1="G";
            field2 = _row_;
            field3=""; field4=.; field5=""; field6=.;
            output rows;
            %do i=1 %to &nvar;
                field1="";
                field2="&&var&i";
                field3=_row_;
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output columns;
            %end;
        end;
        /* process free rows; */
        else if _type_="FREE" then do;
            field1="L";
            field2 = _row_;
            field3=""; field4=.; field5=""; field6=.;
            output rows;
            field1="";
            %if %symexist(rhs) %then %do; field2="&rhs"; %end; %else %do; field2="RHS"; %end;
            field3=_row_;
            field4=1e+40;
            field5=""; field6=.;
            output rhs;
            %do i=1 %to &nvar;
                field1="";
                field2="&&var&i";
                field3=_row_;
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output columns;
            %end;
        end;
        /* process free variable; */
        else if _type_="UNRSTRCT" then do;
            %do i=1 %to &nvar;
                field1="FR";
                field2="BOUNDS";
                field3="&&var&i";
                field4=.;
                field5=""; field6=.;
                if &&var&i ne . then output bounds;
            %end;
        end;
        /* process variable upper bound; */
        else if _type_ in ("UPPERBD", "UPPER") then do;
            %do i=1 %to &nvar;
                field1="UP";
                field2="BOUNDS";
                field3="&&var&i";
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output bounds;
            %end;
        end;
        /* process variable lower bound; */
        else if _type_ in ("LOWERBD", "LOWER") then do;
            %do i=1 %to &nvar;
                field1="LO";
                field2="BOUNDS";
                field3="&&var&i";
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output bounds;
            %end;
        end;
        /* process fixed variable; */
        else if _type_="FIXED" then do;
            %do i=1 %to &nvar;
                field1="FX";
                field2="BOUNDS";
                field3="&&var&i";
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output bounds;
            %end;
        end;
        /* process integer variable; */
        else if _type_ in ("INTEGER", "INT") then do;
            %do i=1 %to &nvar;
                field1="";
                field2="&&var&i";
                field3=_row_;
                field4=&&var&i;
                field5=""; field6=.;
                if &&var&i ne . then output integer;
            %end;
        end;
        /* process binary variable; */
        else if _type_="BINARY" then do;
            %do i=1 %to &nvar;
                field1="BV";
                field2="BOUNDS";
                field3="&&var&i";
                field4=.;
                field5=""; field6=.;
                if &&var&i ne . then output bounds;
            %end;
        end;
        /* process unsupported types */
        else if _type_ in ("SOSEQ","SOSLE","BASIC","PRICESEN","RHSSEN") then do;
            call symput("unsupported", "1");
        end;
    run;

    /* delete empty optional sections */
    %let count=0;
    proc sql noprint;
        select count(*) into :count from rhs;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete rhs;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from ranges;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete ranges;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from bounds;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete bounds;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from sos;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete sos;
        quit;
    %end;

    proc sql noprint;
        select count(*) into :count from integer;
    quit;
    %if &count=0 %then %do;
        proc datasets nolist;
            delete integer;
        quit;
    %end;

    /* move constraint matrix coefficients related with integer variables to separate dataset */
    %if %sysfunc(exist(integer)) %then %do;
        proc sql noprint;
            create table columns2 as
                select distinct * from columns where field2 not in (select field2 from integer);
            create table columns3 as
                select distinct * from columns where field2 in (select field2 from integer);
        quit;

        proc datasets nolist;
            delete integer;
            change columns3=integer;
            delete columns;
            change columns2=columns;
        quit;

        proc sql noprint;
            select count(*) into :count from integer;
        quit;
        %if &count=0 %then %do;
            proc datasets nolist;
                delete integer;
            quit;
        %end;
    %end;

    proc sort data=columns out=columns2;
        by field2;
    run;
    proc datasets nolist;
        delete columns;
        change columns2=columns;
    quit;

    %if %sysfunc(exist(integer)) %then %do;
        proc sort data=integer out=integer2;
            by field2;
        run;
        proc datasets nolist;
            delete integer;
            change integer2=integer;
        quit;
    %end;

    /* generate MPS format dataset */
    options notes;
    data &mps;
        set name
            rows_title rows
            columns_title columns
            %if %sysfunc(exist(integer)) %then intorg integer intend;
            %if %sysfunc(exist(rhs)) %then rhs_title rhs;
            %if %sysfunc(exist(ranges)) %then ranges_title ranges;
            %if %sysfunc(exist(bounds)) %then bounds_title bounds;
            %if %sysfunc(exist(sos)) %then sos_title sos;
            endata;
    run;
    options nonotes;

    /* delete temporary datasets */
    proc datasets nolist;
        delete lp_contents;
        delete name;
        delete rows_title;
        delete columns_title;
        delete rhs_title;
        delete ranges_title;
        delete bounds_title;
        delete sos_title;
        delete endata;
        delete intorg;
        delete intend;
        delete rows;
        delete columns;
        %if %sysfunc(exist(integer)) %then %do; delete integer; %end;
        %if %sysfunc(exist(rhs)) %then %do; delete rhs; %end;
        %if %sysfunc(exist(ranges)) %then %do; delete ranges; %end;
        %if %sysfunc(exist(bounds)) %then %do; delete bounds; %end;
        %if %sysfunc(exist(sos)) %then %do; delete sos; %end;
    quit;

    /* if there are unsupported information, issue a warning message */
    %if &unsupported=1 %then %do;
        options nonotes;
        data _null_;
            putlog 'WARNING: Types SOSEQ, SOSLE, BASIC, PRICESEN, RHSSEN are';
            putlog 'WARNING: not currently supported by MPS format dataset.';
            putlog 'WARNING: All information related with these types are discarded.';
        run;
        option &notes;
    %end;
%mend;


    %if ( %qsubstr(&format,1,1)=%str(%') AND %qsubstr(&format,%length(&format),1)=%str(%') ) OR
        ( %qsubstr(&format,1,1)=%str(%") AND %qsubstr(&format,%length(&format),1)=%str(%") )
        %then %do;
        %let format = %qsubstr(&format, 2, %eval(%length(&format)-2));
    %end;
    %let format = %qupcase(&format);

    %if &format=DENSE %then %do;
        %lp2mps_dense(lp=&lp, mps=&mps);
    %end;
    %else %if &format=SPARSE %then %do;
        %lp2mps_sparse(lp=&lp, mps=&mps);
    %end;

options &notes;
%mend;
