/********************************************************************************************************/
/**																									   **/
/**            Programme servant  optimiser le problme pour le plan de transformation, ensuite      **/
/**            crer les tables ncessaires  la prsentation de la solution et  produire dex         **/
/*             diagrammes de GANTT.                                                                    **/
/** 		   Cre le : 																		 	   **/
/**  		   Modifier le : 29/08/09																   **/
/**            Marche  suivre : Excuter au complet le programme en prenant soin de vrifier que le   **/
/**                              PROC OPTMILP  bien fonctionn, c'est--dire dans la fentre "Sortie  **/
/**                              dans la section Solution Summary the OPTMILP procedure sous la        **/
/**                              variable objectif value il y a une valeur autre que zro ou que       **/
/**                              sous la variable objectif statue ce n'est pas inscrit infeasible ou   **/
/**                              que la fentre "Sortie" ne soit pas vide. En fait, vous devez         **/
/**                              regarder que sous la section Solution Summary the OPTMILP procedure   **/
/**      						 il soit inscrit sour Solution Status Optimal_RGap.                    **/
/**							    Si jamais le problme est Infeasible, la marjorit du temps vous      **/
/**                              devez augementer par bonds de 500 J/P les limites soit OP ou INFO.    **/
/**																									   **/
/********************************************************************************************************/

dm log "color error orange reverse" continue; 

libname martin "\\fic1\doc\DP_SAS\PlanTransformationParOptimisation\CLUBSAS";

/***********************************************/
/** Cration des tables dans la bibliothque  **/
/*       du Work pour utilisation             **/
/***********************************************/

data input_test;
	set martin.input_test;
	where duree > 0;
run;

data lienps_test;
	set martin.lienps_test;
run;

data impossibilite;
	set martin.impossibilite;
run;

data debut_force;
	set martin.debut_force;
	keep activite annee_debut;
run;

data debut_force_liee;
	set martin.debut_force_liee;
run;

/***********************************************/
/** Cration des variables qui compte le      **/
/**	 nombre de variables des fichiers         **/
/**	     exports prcdament                 **/
/***********************************************/

data _null_;
	set impossibilite end=fin;
	if fin then call symput('NbAct4',_n_);
run;

data _null_;
	set debut_force_liee end=fin;
	if fin then call symput('NbAct3',_n_);
run;

data _null_;
	set debut_force end=fin;
	if fin then call symput('NbAct2',_n_); 
run;

data _null_;
	set LienPS_test end=fin;
	if fin then call symput('NbLienPS',_n_);
run;

data _null_;
	set input_test end=fin;
	retain MaxDuree 0;
	if duree > MaxDuree then MaxDuree = duree;
	if fin then do; call symput('NbAct',_n_); call symput('MaxDuree',MaxDuree); end;
run;

/***********************************************/
/**     Cration du problme linaire         **/
/**             en format LP                  **/
/***********************************************/

%let NbAnnees = 30; /** Dure maximal du plan de transformation **/
%let CapaciteTotaleInfo = 25000; /** Contrainte de limiteINFO **/
%let CapaciteTotaleOP = 9500; /** Contrainte de limiteOP **/

data martin.modele1_test;
	length _type_ $8. _col_ $20. _row_ $25.;
	keep _type_ _col_ _row_ _coef_;

	format activite $4. activite_liee $4. prealable $4. successeur $4. annee_debut $3. annee_non_debut $3.;

	indicD = 1;

	do i = 1 to &NbAct;  /* &NbAct = nombre de processus cr  partir de la table input_test */
		link input_test;
		do j = 1 to &NbAnnees; /* &NbAnnees = dure maximal du plan de transformation */

			/*****************************/
			/**     fonction objectif   **/
		    /**   maximise la priorit  **/
		    /**      dans le temps      **/
			/*****************************/

			if indicD = 1 then do; _type_ = "max"; indicD = 0; end;
				else _type_ = " ";
			_row_ = "priorite";
			_col_  = compress("Rho_" || activite || put(j,z2.));
			_coef_	= priorite / j;
			output;

			/*******************************************/
			/**           CONTRAINTE                  **/
			/**  variable de dcision dfinie comme   **/
			/**  binaire : dbut de la transformation **/
			/** pour la variable Rho_ et debut,milieu **/
            /**  et fin de la transformation pour la  **/
            /**  variable  gamma_                     **/
			/*******************************************/

			if (i = 1 and j = 1) then do;
				_type_ = "BINARY";
				_col_  = " ";
				_row_  = "Rhobinaire";
				_coef_ = .;
				output;
				indic = 1;
			end;
			_type_ = " ";
			_row_  = "Rhobinaire";
			_col_  = compress("Rho_" || activite || put(j,z2.));
			_coef_ = indic;
			indic  = indic + 1;
			output;

			_type_ = " ";
			_row_  = "Rhobinaire";
			_col_  = compress("Gamma_" || activite || put(j,z2.));
			_coef_ = indic;
			indic  = indic + 1;
			output;

			/*******************************************/
			/**           CONTRAINTE                  **/
			/**  un seul debut de transformation      **/
			/**            par activite               **/
			/*******************************************/

			if j = 1 then do;
				_type_ = "eq";
				_col_ = "_rhs_";
				_coef_ = 1;
				_row_ = "Demarrage_" || activite;
				output;
			end;
			_type_ = " ";
			_col_ = compress("Rho_" || activite || put(j,z2.));
			_coef_ = 1;
			_row_ = "Demarrage_" || activite;
			output;

			/*******************************************/
			/**           CONTRAINTE                  **/
			/** dfinition de la variable indiquant   **/
			/**    si le projet est en cours          **/
			/*******************************************/

			do t = 1 to &MaxDuree;
				if j ge t and t le duree then do;
					if t = 1 then do;
						_type_ = "eq";
						_col_ = "_rhs_";
						_coef_ = 0;
						_row_ = compress("DefGamma_" || activite || put(j,z2.));
						output;
						_type_ = " ";
						_col_ = compress("Gamma_" || activite || put(j,z2.));
						_coef_ = -1;
						_row_ = compress("DefGamma_" || activite || put(j,z2.));
						output;	
				     end;
					_type_ = " ";
					_col_ = compress("Rho_" || activite || put(j-t+1,z2.));
					_coef_ = 1;
					_row_ = compress("DefGamma_" || activite || put(j,z2.));
					output;	
				end;
			end;

			/*******************************************/
			/**           CONTRAINTE  LIMITE INFO     **/
			/**    Contrainte de CoutTotalInfo,       **/
			/** c'est--dire contrainte par anne     **/
			/**       des jour/personnes DSI           **/
			/*******************************************/	
/* NOTE : L'anne 3 est change manuellement  23000 j/p   partir du fichierinput_21d.xls*/

		if j = 3 then do;
				if i = 1 then do;
					_type_ = "le";
					_col_  = "_rhs_";
					_coef_ = 25000;
					_row_  = compress("LimiteInfo_" || put(j,z2.));
					output;
				end;
				_type_ = " ";
				_col_  = compress("Gamma_" || activite || put(j,z2.));
				_coef_ = rnDSI  / duree  ;
				_row_  = compress("LimiteInfo_" || put(j,z2.));
				output;
			end;

			else do;
				if i = 1 then do;
					_type_ = "le";
					_col_  = "_rhs_";
					 _coef_ = &CapaciteTotaleInfo ;
					_row_  = compress("LimiteInfo_" || put(j,z2.));
					output;
				end;
				_type_ = " ";
				_col_  = compress("Gamma_" || activite || put(j,z2.));
				_coef_ = rnDSI  / duree  ;
				_row_  = compress("LimiteInfo_" || put(j,z2.));
				output;
			end;

			/*******************************************/
			/**           CONTRAINTE LIMITE OP        **/
			/**    Contrainte de CapaciteTotaleOP     **/
			/** c'est--dire contrainte par anne     **/
			/**      des jour/personnes OP            **/
			/*******************************************/	


			if j = 3 then do; * anne 2010-2011 *;
				if i = 1 then do;
					_type_ = "le";
					_col_  = "_rhs_";
					*_coef_ = &CapaciteTotaleOP ;
					 _coef_ = 10000 ;
					_row_  = compress("LimiteOP_" || put(j,z2.));
					output;
				end;
				_type_ = " ";
				_col_  = compress("Gamma_" || activite || put(j,z2.));
				_coef_ = rnOP  / duree  ;
				_row_  = compress("LimiteOP_" || put(j,z2.));
				output;
			end;

			else if j = 4 then do;
				if i = 1 then do;
					_type_ = "le";
					_col_  = "_rhs_";
					_coef_ = &CapaciteTotaleOP ;
					* _coef_ = 8500 ;
					_row_  = compress("LimiteOP_" || put(j,z2.));
					output;
				end;
				_type_ = " ";
				_col_  = compress("Gamma_" || activite || put(j,z2.));
				_coef_ = rnOP  / duree  ;
				_row_  = compress("LimiteOP_" || put(j,z2.));
				output;
			end;

			else do;
				if i = 1 then do;
					_type_ = "le";
					_col_  = "_rhs_";
					 _coef_ = &CapaciteTotaleOP ;
					_row_  = compress("LimiteOP_" || put(j,z2.));
					output;
				end;
				_type_ = " ";
				_col_  = compress("Gamma_" || activite || put(j,z2.));
				_coef_ = rnOP  / duree  ;
				_row_  = compress("LimiteOP_" || put(j,z2.));
				output;
			end;  
			

		end;	
	end;

	/*******************************************/
	/**           CONTRAINTE                  **/
	/**     Contrainte de Liens pralables,   **/
	/** c'est--dire les liens pralables et  **/
	/**  successeur qui a en entre la table  **/
	/**              LienPS_test              **/
	/*******************************************/	

	do l = 1 to &NbLienPS;
		link LienPS_test;
		do j = 1 to &NbAnnees; 
			_type_ = "ge";
			_col_  = "_rhs_";
			_row_  = compress("LienPS_" || prealable || successeur || put(j,z2.));
			_coef_ = 0;
			output;	
			_type_ = " ";
			_col_  = compress("Rho_" || successeur || put((j+AnAvant),z2.));
			_row_  = compress("LienPS_" || prealable || successeur || put(j,z2.));
			_coef_ = -1;
			output;
			do k = 1 to j;
				_type_ = " ";
				_col_  = compress("Rho_" || prealable || put(k,z2.));
				_row_  = compress("LienPS_" || prealable || successeur || put(j,z2.));
				_coef_ = 1;
				output;
			end;
		end;		

		do m = 1 to AnAvant;
			_type_ = "eq";
			_col_  = "_rhs_";
			_row_  = compress("LienPSForce_" || prealable || successeur || put(m,z2.));
			_coef_ = 0;
			output;	
			do n = 1 to m;
			_type_ = " ";
			_col_  = compress("Rho_" || successeur || put(n,z2.));
			_row_  = compress("LienPSForce_" || prealable || successeur || put(m,z2.));
			_coef_ = 1;
			output;
			end;
		end;
		
	end;

	/*******************************************/
	/**           CONTRAINTE                  **/
	/**     Contrainte de dbut forc des ,   **/
	/** activits qui sont lie dans l'onglet **/
	/**  lienACT et qui a en entre la table  **/
	/**         debut_force_liee              **/
	/*******************************************/	

	do i = 1 to &NbAct3;
		link debut_force_liee;
		do j = 1 to (&NbAnnees) ;

			if j = &NbAnnees then do;
				_type_ = "eq";
				_col_ = "_rhs_";
				_row_ =  compress("InitialAct_" || activite || put(j,z2.));
				_coef_ = 0 ;
				output;
				_type_ = "";
				_col_ = compress("Rho_" || activite || put(j,z2.));
				_row_ = compress("InitialAct_" || activite || put(j,z2.) );
				_coef_ = 1 ;
				output;	
			end;

			if j = 1 then do;
				_type_ = "eq";
				_col_ = "_rhs_";
				_row_ =  compress("InitialActLiee_" || activite_liee || put(j,z2.));
				_coef_ = 0 ;
				output;
				_type_ = "";
				_col_ = compress("Rho_" || activite_liee || put(j,z2.));
				_row_ = compress("InitialActLiee_" || activite_liee || put(j,z2.));
				_coef_ = 1 ;
				output;
			end;

			_type_ = "eq";
			_col_  = "_rhs_";
			_row_  = compress("DebutForceLiee_" || activite || activite_liee || put(j,z2.));
			_coef_ = 0;
			output;
			_type_ = "";
			_col_  = compress("Rho_" || activite || put(j,z2.));
			_row_  = compress("DebutForceLiee_" || activite || activite_liee || put(j,z2.));
			_coef_ = -1;
			output;
			_type_ = "";
			_col_  = compress("Rho_" || activite_liee || put(j+(duree),z2.));
			_row_  = compress("DebutForceLiee_" || activite || activite_liee || put(j,z2.));
			_coef_ = 1;         
			output;
		

		end;
	end;

	/*******************************************/
	/**           CONTRAINTE                  **/
	/**  Contrainte de dbut forc qui force  **/
	/** le dbut de certains processus et qui **/
	/**   a en entre la table debut_force    **/
	/*******************************************/	

	do i = 1 to &NbAct2;
		link debut_force;
		_type_ = "eq";
		_col_  = "_rhs_";
		_row_  = compress("DebutForce_" || activite);
		_coef_ = 1;
		output;
		_type_ = "";
		_col_  = compress("Rho_" || activite || annee_debut);
		_row_  = compress("DebutForce_" || activite);
		_coef_ = 1;
		output;
	end;

	/*******************************************/
	/**           CONTRAINTE                  **/
	/** Contrainte d'impossibilit qui force  **/
	/** l'impossibilit de certains processus **/
	/**      et qui a en entre la table      **/
	/**           impossibilite               **/
	/*******************************************/

	do i = 1 to &NbAct4;
		link impossibilite;
		if i = 1 then do ;
		_type_ = "eq";
		_col_  = "_rhs_";
		_row_  = compress("ImpossibiliteAct_");
		_coef_ = 0;
		output;
		end;
		_type_ = "";
		_col_  = compress("Rho_" || activite || annee_non_debut);
		_row_  = compress("ImpossibiliteAct_");
		_coef_ = 1;
		output;
	end;

	/*******************************************/
	/**  Emplacement des fichiers utiliss    **/
	/*******************************************/

	input_test : 	set input_test;
		return;

	LienPS_test : 	set LienPS_test;
		return;

	debut_force : set debut_force;
		return;

	debut_force_liee : set debut_force_liee;
		return;

	impossibilite : set impossibilite;
		return;
run;

proc sort data=martin.modele1_test NODUPKEY;
	by _row_  _type_ _coef_  _col_;
run;

proc sort data=martin.modele1_test;
	by _row_ descending _type_;
run;

/***********************************************/
/**     Cration du problme linaire         **/
/**             en format MPS                 **/
/***********************************************/

%lp2mpsd(1.4,lp=martin.modele1_test, mps=martin.modele2_test, format=SPARSE);

/***********************************************/
/**       Rsolution du problme linaire     **/
/***********************************************/

PROC OPTMILP DATA=martin.modele2_test  
			 PRIMALOUT=martin.solution1_test PRINTLEVEL=1;
RUN;

data s_test;
	set martin.solution1_test;
run;

/***********************************************/
/**     Prparation des rsultats             **/
/***********************************************/ 

data pasjuste(drop=_value_);
	set s_test;
	length test 3.0;
	test = abs(round(_value_,1));
	
run;

data s_test;
	set pasjuste;
	rename test = _value_;
run;

data R_test;
	set S_test;
	where _VAR_ like 'GAMMA%';
	format activite $4. an 8. IndicEnCours 8.;
	an = substr(_VAR_,11,2);
	activite = substr(_VAR_,7,4);
	IndicEnCours = _value_;
	keep activite an IndicEnCours;
run;

proc sql;
	create table RessInfo_Test as
	select b._var_, a.field4, b._value_
	from martin.modele2_test as a, S_test as b
	where a.field2 = b._var_
		and (field3 like "LIMITEINFO_%");
quit;

data RessInfo_Test;
	set RessInfo_Test;
	format activite $4. an 8.;
run;

data RessInfo_Test;
	set RessInfo_Test;
	an = substr(_VAR_,11,2);
	activite = substr(_VAR_,7,4);
	CoutRessInfo = _value_ * field4;
	*CoutRessInfo = _value_ * field4;
	keep activite an CoutRessInfo _value_;
run;

proc sql;
	create table R_test as
	select *
	from R_test as a , RessInfo_Test as b
	where a.activite = b.activite and a.an = b.an;
quit;

proc sql;
	create table R_test as
	select a.*, b.duree, b.RnOP, b.RnDSI
	from R_test as a , input_test as b
	where a.activite = b.activite;
quit;

/***********************************************/
/** Ici on fusionne les processus A,B,C ....  **/
/** ensemble. Cependant il y a des processus  **/
/**  A,B,C... qui doivent rester distincts et **/
/** c'est dans ce DATA que nous l'inscrivont  **/
/** Regarder l'exemple de P13 pour comprendre **/
/***********************************************/

data R_test;
	set R_test;
	format activite3 $4.;

	CoutTotalDSI = (423.55*RnDSI/duree)*_value_;
	CoutTotalop = (366.45*RnOP/duree)*_value_;
	CoutTotal = CoutTotalDSI + CoutTotalop;

	activite3 = substr(activite,1,3);
	if activite = "P13A" or activite = "P13B" or activite = "P13C" /*or activite = "P13D" */ then activite3 = "P13X";
	if activite = "P13D" then activite3 = "P13Y"; /* Attention P13D devient P13E */
	*if activite = "FD1B" or activite = "FD1C" then activite3 = "FDX";
	if activite = "FD1B" then activite3 = "FD1B";
	if activite = "FD1C" then activite3 = "FD1C";
	if activite = "FD1D" or activite = "FD1E" then activite3 = "FDY";
	if activite = "FG1A" or activite = "FG1B" then activite3 = "FGX";
	if activite = "FG1C" or activite = "FG1D" then activite3 = "FGY";
	if activite = "BF1A" or activite = "BF1B" or activite = "BF1C" or activite = "BF1D" then activite3 = "BFX";
	if activite = "BF1E" or activite = "BF1F" or activite = "BF1G" or activite = "BF1H" then activite3 = "BFY";
	*if activite = "P16A" or activite = "P17A" or activite = "P18A" then activite3 = "P16";
	if activite = "FB1A" then activite3 = "FBX";
	if activite = "FB1B" then activite3 = "FBY";
	if activite = "FD1A" then activite3 = "FDA";
 	if activite = "FD1D" then activite3 = "F1X";
	if activite = "FD1E" then activite3 = "F1Y";
	if activite = "BS1X" or activite = "BS1Y" then activite3 = "BSX";
	if activite = "BS1B" then activite3 = "BSY";
	if activite = "BN1B" then activite3 = "BN1B";
	if activite = "BT1A" or activite = "BT1B" then activite3 = "BT1";
	if activite = "BT1C" then activite3 = "BT1C";
	if activite = "P05A" or activite = "P05B"  then activite3= "P05X";
	if activite = "P05C" or activite = "P05D" or activite = "P05E" then activite3= "P05Y";
	if activite = "BU1A" or activite = "BU1B" or activite = "BU1C" or activite = "BU1D" then activite3 = "P17X";

	if activite3="BC1" then delete;

run;

/***********************************************/
/**     Prparation des rsultats             **/
/***********************************************/ 

data r_test2;
	set r_test;
	*format CoutRessOp 20.2;
	*format CoutRessOp2 20.2;
 	*CoutRessOp = round((_value_/Duree),0.2) * rnop;
	CoutRessOp = _value_ * rnop / Duree;
run;

proc sort data=r_test2;
	by activite coutressop;
run;
/*
data r_test2;
	set r_test2;
	by activite coutressop an;
	if first.coutressop then coutressop2 = coutressop;
	else                   coutressop2 = 0;
run;*/

proc sql;
	create table R_test2 as
	select *
	from R_test2 as a, martin.nom as b
	where a.activite3 = b.variable;
quit;

data R_test2;
	set R_test2;

	if CodeCouleur = 1 then tri = 1;
	if CodeCouleur = 2 then tri = 2;
	if CodeCouleur = 3 then tri = 3;
	if CodeCouleur = 14 then tri = 4;
	if CodeCouleur = 8 then tri = 5;
	if CodeCouleur = 6 then tri = 6;
	part = 1;
run;

/* afin de trouver la derniere annee du plan */
data max;
	set r_Test2(where=(couttotal > 0));

	part=1;
run;

proc sort data=max;
	by part an;
run;

data max2(keep=part an);
	set max;
	by part an;
	if last.part;	
run;

data r_test2(drop=part);
	merge r_test2(in=a) max2(in=b rename=(an=max));
	by part;
run;

/***********************************************/
/**      Tableau final des rsultats          **/
/***********************************************/ 

data martin.tableau;
	set r_test2;
run;

/*****  Sommation du nombre de jours personne par activite *****/

proc sort data=r_test2;
	by activite an;
run;

data temp;
	set r_Test2;
	by activite an;
	if last.activite;
run;

proc summary data=temp;
	class activite3;
	var rndsi;
	output out=stemp(keep=activite3 jp) sum=jp;
run;

proc summary data=temp;
	class activite3;
	var rnop;
	output out=stemp2(keep=activite3 jp2) sum=jp2;
run;

/***********************************************/
/**  Prparation pour le diagramme GANTT      **/
/***********************************************/ 

proc sort data=R_test2;
	by activite3;
run;

data r2_test;
	merge r_Test2(in=a) stemp(in=b);
	by activite3;
	if a;
run;

proc sort data=r2_test;
	by prealable;
run;

data to_chart;
	set R2_test;
	where IndicEnCours between 0.99 and 1.1;
	*where IndicEnCours =1;
	
	by prealable;
	format cout $30.;
	
	pat = CodeCouleur;
	*Cout = compress(jp || "J/P_" ||  CoutTotal || "$");
	Cout = compress(put(jp,30.2) || " J/P");
	if (variable = "P8A" or variable = "P8B") then tri2 = "P08";
		else tri2 = variable;
	retain andeb anfin;
	if first.prealable then andeb = an - 1 + 2008;
	if last.prealable then do; anfin = an + 2008; duree = anfin - andeb; output; end;
run;
data martin.to_chart;
	set to_chart;
run;

proc sort data=to_chart;
	by prealable;
run;

proc sort data=R2_Test(keep=activite prealable) out=rr nodupkey;
	by activite;
	run;

proc sql;
	create table lienPS_test2 as
	select a.prealable as cle_prealable,a.successeur,b.prealable
	from lienps_test as a, rr as b
	where a.prealable = b.activite;
quit;

proc sql;
	create table lienPS_test2 as
	select a.prealable, a.cle_prealable, a.successeur as cle_successeur, b.prealable as successeur
	from lienps_test2 as a, rr as b
	where a.successeur = b.activite;
quit;

proc sort data=lienps_test2(where=(not(prealable=successeur))) out=lienps_test2 nodupkey;
	by prealable successeur;
	run;

data _null_;
	set lienps_test2 end=fin;
	by prealable;
	retain max 0 compteur 0;
	if first.prealable then compteur = 1;
		else compteur = compteur + 1;
	if compteur > max then max = compteur;
	if fin then call symput('NbSucc',compress(max));
run;

proc transpose data=lienps_test2 out=lienps_test2 prefix=succ;
	var successeur;
	by prealable;
	run;

proc sort data=to_chart;
	by prealable;
run;

data to_chart;
	merge to_chart
		  lienps_test2;
	by prealable;
	drop _label_;
run;

PROC SORT DATA=TO_CHART;
	BY TRI;
RUN;

signoff;

/***********************************************/
/**      Procdure du diagramme GANTT         **/
/***********************************************/ 

data labels;
_y=-1;
_lvar="Cout";
_xvar="Anfin";
_jlabel = "L";
_hlabel = 0.85;
_YOFFSET = -0.15;
run;

title1 color=black f="Arial/Bold" h=3 "Planification de la transformation";

title2 color=black f="Arial" h=1 "                  ROUGE = Projets TI | VERT = Processus de services aux citoyens | BLEU  = Fonctions communes | BLEU RAYE EN X = Fonctions complementaires processus | ORANGE = Processus de gestion et de soutien | GRIS = Autres lments suivis" ;

title3 color=black f="Arial" h=1 "%sysfunc(date(),DDMMYY.)";

proc gantt graphics data=to_chart labdata=labels;
	chart /	combine
			compress
			scale = 9
			s_start = andeb
			s_finish = anfin
			duration = duree
			padding = dtyear
			mindate = 2008
			pattern = pat
			activity = prealable
			successor = (succ1-succ&NbSucc)
			Cprec = black wprec=2
			nojobnum 
			nolegend;	      
	id prealable;
run;
signoff;



data to_chart2;
	set to_chart;
	andeb = anfin - 1;
	If tri =  1 then cat  = "TI";
	If tri =  2 then cat  = "SC";
	If tri =  3 then cat  = "FC";
	If tri =  4 then cat  = "FCP";
	If tri =  5 then cat  = "GS";
	If tri =  6 then cat  = "Au.";
run;

PROC SORT DATA=TO_CHART2;
	BY TRI andeb;
RUN;


/***********************************************/
/**      Procdure du diagramme GANTT  autre  **/
/** qui prsente seulement l'anne de fin     **/
/***********************************************/ 

title1 color=black f="Arial/Bold" h=3 "Planification de la transformation - Carte de livraison";

title2 color=black f="Arial" h=1 "                  ROUGE = Projets TI | VERT = Processus de services aux citoyens | BLEU  = Fonctions communes | BLEU RAYE EN X = Fonctions complementaires processus | ORANGE = Processus de gestion et de soutien | GRIS = Autres lments suivis" ;

title3 color=black f="Arial" h=1 "%sysfunc(date(),DDMMYY.)";

proc gantt graphics data=to_chart2 ;
	chart /	combine
			compress
			scale = 9
			s_start = andeb 
			s_finish = anfin
			duration = duree
			padding = dtyear
			mindate = 2008
			pattern = pat
			activity = prealable
			successor = (succ1-succ&NbSucc)
			Cprec = black wprec=5
			/*HCONNECT*/ /*Cette fonction permet de faire des lignes qui part de la fin du projet jusqu' l'anne 1*/
			/*ref= 2021 to 2022 by 1*/ /* Cette fonction mettera des lignes verticales  l'anne 2021 et 2022 */
			zone=cat onezoneval /* Cette fonction met un lign honrizontale  chaque nouvelle catgorie (cat)*/
			nojobnum 
			nolegend;	      
	id prealable;
run;
